package com.cusc.nirvana.user.rnr.openapi.interceptor;

import com.alibaba.fastjson.JSON;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.openapi.common.Constants;
import com.cusc.nirvana.user.rnr.openapi.config.OpenapiConfig;
import com.cusc.nirvana.user.rnr.openapi.constants.AuthConstant;
import com.cusc.nirvana.user.rnr.openapi.util.SignUtils;
import com.cusc.nirvana.user.rnr.openapi.util.ThreadLocalUtil;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

/**
 * Description: 鉴权拦截器
 * <p>
 * 1. 从RequestHead中获取开放平台租户ID（TENANT_NO），通过开放平台租户ID获取对应的实名制系统租户ID（rnrTenantRelation）
 * 2. 通过入参中的组织ID（orgId）获取组织对应的实名制系统租户ID（rnrTenant）
 * 3. 校验 实名制系统租户ID（rnrTenant） 是否与 实名制系统租户ID（rnrTenantRelation）是否相同，
 * 不相同，拦截
 * 相同，将参数中的用户ID，实名制系统租户ID放入ThreadLocal中，后续使用
 * <br />
 * <p>
 * CreateDate 2022-05-06 14:36
 *
 * @author caow
 **/
public class AuthInterceptor implements HandlerInterceptor {

    private static final Logger LOGGER = LoggerFactory.getLogger(AuthInterceptor.class);

    @Autowired
    OpenapiConfig openapiConfig;

    @Value("${auth.userId:OpenApi}")
    private String userId;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {

        String requestUri = request.getRequestURI();

        Response checkResponse = signCheck(request);

        if (!checkResponse.isSuccess()) {
            LOGGER.info("AuthInterceptor url:{}, sign check failed, {}.", requestUri, checkResponse.getMsg());
            returnJson(response,
                    JSON.toJSONString(Response.createError(checkResponse.getMsg(),
                            ResponseCode.INVALID_DATA.getCode())));
            return false;
        }

        ThreadLocalUtil.set(AuthConstant.TENANT_NO_NAME, openapiConfig.getTenantNo());
        ThreadLocalUtil.set(AuthConstant.USER_ID_NAME, userId);

        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler,
                                Exception ex) {
        ThreadLocalUtil.removeByKey(AuthConstant.USER_ID_NAME);
        ThreadLocalUtil.removeByKey(AuthConstant.TENANT_NO_NAME);
    }

    /**
     * 签名校验
     * @param request
     * @return
     */
    private Response signCheck(HttpServletRequest request) {
        String sign = request.getHeader(Constants.SIGN);
        String appId = request.getHeader(Constants.APP_ID);
        String nonceStr = request.getHeader(Constants.NONCE_STR);
        String timeStamp = request.getHeader(Constants.TIMESTAMP);
        String version = request.getHeader(Constants.VERSION);

        if(StringUtils.isBlank(sign)){
            return Response.createError("请求Header中缺少SIGN");
        }
        if(StringUtils.isBlank(appId)){
            return Response.createError("请求Header中缺少APPID");
        }
        if(StringUtils.isBlank(nonceStr)){
            return Response.createError("请求Header中缺少NONCE_STR");
        }
        if(StringUtils.isBlank(timeStamp) || !StringUtils.isNumeric(timeStamp)){
            return Response.createError("请求Header中缺少TIMESTAMP或TIMESTAMP格式错误");
        }
        if(StringUtils.isBlank(version)){
            return Response.createError("请求Header中缺少VERSION");
        }

        Map<String, String> map = Maps.newHashMapWithExpectedSize(4);
        map.put(Constants.APP_ID, appId);
        map.put(Constants.TIMESTAMP, timeStamp);
        map.put(Constants.NONCE_STR, nonceStr);
        map.put(Constants.VERSION, version);

        long currentTimeMillis = System.currentTimeMillis();
        long apiTime = Long.parseLong(timeStamp);
        if(currentTimeMillis > apiTime && (currentTimeMillis - apiTime) >  openapiConfig.getTimeOut()){
            return Response.createError("TIMESTAMP超时，已超过"+openapiConfig.getTimeOut()/1000+"秒");
        }

        boolean valid = SignUtils.getInstance().isValid(sign, map, openapiConfig.getAppSecret());
        if(!valid){
            return Response.createError("签名校验失败，请确认签名信息");
        }
        return Response.createSuccess();
    }

    private void returnJson(HttpServletResponse response, String json) {
        response.setCharacterEncoding("utf-8");
        response.setContentType("application/json; charset=utf-8");
        PrintWriter writer = null;
        try {
            writer = response.getWriter();
            writer.write(json);
        } catch (IOException e) {
            LOGGER.error("AuthInterceptor 响应json 异常： {}", e);
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }
}
